@@ -31,6 +31,7 @@ import Data.Char (isDigit, isAscii, isLetter)
 import qualified Data.Map as M
 import qualified Data.Text as T
 import Data.Text (Text)
+import Data.Ratio ((%))
 import Data.Maybe (catMaybes, fromJust, mapMaybe)
 import Text.Parsec hiding (label)
 import Text.Parsec.Error
@@ -52,7 +53,27 @@ type TP = Parser
 -- The parser
 
 expr1 :: TP Exp
-expr1 = choice
+expr1 = do
+  e <- expr2
+  -- check for primes and add them as a subscript
+  -- in TeX ' is shorthand for ^{\prime}
+  primes <- many (char '\'')
+  let getPrimes cs = case cs of
+                       "" -> ""
+                       "'" -> "\x2032"
+                       "''" -> "\x2033"
+                       "'''" -> "\x2034"
+                       "''''" -> "\x2057"
+                       _ -> "\x2057" ++ getPrimes (drop 4 cs)
+  ignorable
+  case getPrimes primes of
+    "" -> return e
+    cs -> return $ case e of
+                     ESub b sub -> ESubsup b sub (ESymbol Pun (T.pack cs))
+                     _ -> ESuper e (ESymbol Pun (T.pack cs))
+
+expr2 :: TP Exp
+expr2 = choice
           [ inbraces
           , variable
           , number
@@ -62,7 +83,7 @@ expr1 = choice
           , enclosure
           , hyperref
           , command
-          ] <* ignorable
+          ]
 
 -- | Parse a formula, returning a list of 'Exp'.
 readTeX :: Text -> Either Text [Exp]
@@ -208,6 +229,7 @@ command = try $ do
     , mathop c
     , phantom c
     , boxed c
+    , cancel c
     , binary c
     , genfrac c
     , substack c
@@ -233,7 +255,7 @@ operatorname = do
     ctrlseq "operatorname"
     -- these are slightly different but we won't worry about that here...
     convertible <- (char '*' >> spaces >> return True) <|> return False
-    tok' <- texToken
+    tok' <- texSymbol <|> braces (manyExp expr2) <|> texChar
     tok'' <- (EMathOperator <$> (expToOperatorName tok'))
            <|> return (EStyled TextNormal [tok'])
     return (tok'', convertible)
@@ -338,7 +360,7 @@ many1Exp :: TP Exp -> TP Exp
 many1Exp = manyExp' True
 
 inbraces :: TP Exp
-inbraces = braces (manyExp expr)
+inbraces = braces (ignorable *> manyExp expr)
 
 texToken :: TP Exp
 texToken = texSymbol <|> inbraces <|> texChar
@@ -408,11 +430,7 @@ delimited = do
 
 delimitedImplicit :: TP Exp
 delimitedImplicit = try $ do
-  (opent, closer) <-
-           (("(", symbol ")") <$ symbol "(")
-       <|> (("[", symbol "]") <$ symbol "[")
-       <|> (("|", symbol "|") <$ symbol "|")
-       <|> (("\x2016", "\x2016" <$ ctrlseq "rVert") <$ ctrlseq "lVert")
+  (opent, closer) <- (("\x2016", "\x2016" <$ ctrlseq "rVert") <$ ctrlseq "lVert")
   contents <- concat <$>
               many (try $ ((:[]) . Left  <$> middle)
                       <|> (map Right . unGrouped <$>
@@ -643,6 +661,14 @@ boxed :: Text -> TP Exp
 boxed "\\boxed" = EBoxed <$> texToken
 boxed _ = mzero
 
+cancel :: Text -> TP Exp
+cancel c = do
+  case c of
+    "\\cancel"  -> ECancel ForwardSlash <$> texToken
+    "\\bcancel" -> ECancel BackSlash <$> texToken
+    "\\xcancel" -> ECancel XSlash <$> texToken
+    _           -> mzero
+
 text :: Text -> TP Exp
 text c = do
   op <- maybe mzero return $ M.lookup c textOps
@@ -702,17 +728,28 @@ xspace "\\mspace" =
        _                     -> mzero
 xspace "\\hspace" = do
   braces $ do
-    len <- many1 digit
-    scaleFactor <-
+    neg <- option 1 $ (-1) <$ char '-'
+    as <- option "" $ many1 digit
+    bs <- option "" $ char '.' *> many1 digit
+    let denominator = 10^(length bs)
+    as' <- if null as then pure 0 else stringToInteger as
+    bs' <- if null bs then pure 0 else stringToInteger bs
+    let numerator = (as' * denominator) + bs'
+    let n = neg * (numerator % denominator)
+    scaleFactor <- lexeme $
            1      <$ (string "em")
       <|> (1/12)  <$ (string "pt")
       <|> 6       <$ (string "in")
       <|> (50/21) <$ (string "cm")
-    case reads len of
-       ((n :: Integer,[]):_) -> return $ ESpace (fromIntegral n * scaleFactor)
-       _                     -> mzero
+    return $ ESpace (n * scaleFactor)
 xspace _ = mzero
 
+stringToInteger :: String -> TP Integer
+stringToInteger s =
+  case reads s of
+    ((n :: Integer, []):_) -> pure n
+    _ -> fail $ "Could not read " <> s <> " as Integer."
+
 mathop :: Text -> TP Exp
 mathop c =
   case c of
@@ -862,6 +899,7 @@ tSymbol sym =
          (\t -> EUnder False t acc) <$> texToken
        Just acc@(ESymbol TOver _) ->
          (\t -> EOver False t acc) <$> texToken
+       Just (ESymbol Alpha t) -> return $ EIdentifier t -- see #256
        Just x  -> return x
        Nothing
          | sym == "\\mod" -> do
@@ -976,6 +1014,7 @@ textCommands = M.fromList
   , ("\\v", option "v" $ try $ hacek <$> tok)
   , ("\\u", option "u" $ try $ breve <$> tok)
   , ("\\ ", return " ")
+  , ("\\quad", return "\x2000")
   ]
 
 parseC :: TP Text
